@@ -2,10 +2,24 @@
 extern crate clap;
 
 use gio::prelude::*;
-use gio::{
-    Cancellable, FileCreateFlags, FileExt, InputStream, OutputStream, UnixInputStream,
-    UnixOutputStream,
-};
+use gio::{Cancellable, FileCreateFlags, FileExt, InputStream, OutputStream};
+
+#[cfg(unix)]
+use gio::{UnixInputStream, UnixOutputStream};
+
+#[cfg(windows)]
+mod windows_imports {
+    pub use gio_sys::{GInputStream, GOutputStream};
+    pub use glib::translate::*;
+    pub use glib_sys::gboolean;
+    pub use libc::c_void;
+    pub use std::io;
+    pub use std::os::windows::io::AsRawHandle;
+}
+
+#[cfg(windows)]
+use self::windows_imports::*;
+
 use librsvg::rsvg_convert_only::{LegacySize, PathOrUrl};
 use librsvg::{CairoRenderer, Color, Loader, Parse, RenderingError};
 use once_cell::unsync::OnceCell;
@@ -21,10 +35,10 @@ impl std::fmt::Display for Error {
     }
 }
 
-impl From<cairo::Status> for Error {
-    fn from(s: cairo::Status) -> Self {
+impl From<cairo::Error> for Error {
+    fn from(s: cairo::Error) -> Self {
         match s {
-            cairo::Status::InvalidSize => Self(String::from(
+            cairo::Error::InvalidSize => Self(String::from(
                 "The resulting image would be larger than 32767 pixels on either dimension.\n\
                 Librsvg currently cannot render to images bigger than that.\n\
                 Please specify a smaller size.",
@@ -154,8 +168,11 @@ impl ResizeStrategy {
 
 enum Surface {
     Png(cairo::ImageSurface, OutputStream),
+    #[cfg(system_deps_have_cairo_pdf)]
     Pdf(cairo::PdfSurface, Size),
+    #[cfg(system_deps_have_cairo_ps)]
     Ps(cairo::PsSurface, Size),
+    #[cfg(system_deps_have_cairo_svg)]
     Svg(cairo::SvgSurface, Size),
 }
 
@@ -165,8 +182,11 @@ impl Deref for Surface {
     fn deref(&self) -> &cairo::Surface {
         match self {
             Self::Png(surface, _) => &surface,
+            #[cfg(system_deps_have_cairo_pdf)]
             Self::Pdf(surface, _) => &surface,
+            #[cfg(system_deps_have_cairo_ps)]
             Self::Ps(surface, _) => &surface,
+            #[cfg(system_deps_have_cairo_svg)]
             Self::Svg(surface, _) => &surface,
         }
     }
@@ -190,6 +210,7 @@ impl Surface {
         Ok(Self::Png(surface, stream))
     }
 
+    #[cfg(system_deps_have_cairo_pdf)]
     fn new_for_pdf(size: Size, stream: OutputStream) -> Result<Self, Error> {
         let surface = cairo::PdfSurface::for_stream(size.w, size.h, stream.into_write())?;
         if let Some(date) = metadata::creation_date()? {
@@ -198,17 +219,35 @@ impl Surface {
         Ok(Self::Pdf(surface, size))
     }
 
+    #[cfg(not(system_deps_have_cairo_pdf))]
+    fn new_for_pdf(_size: Size, _stream: OutputStream) -> Result<Self, Error> {
+        Err(Error("unsupported format".to_string()))
+    }
+
+    #[cfg(system_deps_have_cairo_ps)]
     fn new_for_ps(size: Size, stream: OutputStream, eps: bool) -> Result<Self, Error> {
         let surface = cairo::PsSurface::for_stream(size.w, size.h, stream.into_write())?;
         surface.set_eps(eps);
         Ok(Self::Ps(surface, size))
     }
 
+    #[cfg(not(system_deps_have_cairo_ps))]
+    fn new_for_ps(_size: Size, _stream: OutputStream, _eps: bool) -> Result<Self, Error> {
+        Err(Error("unsupported format".to_string()))
+    }
+
+    #[cfg(system_deps_have_cairo_svg)]
     fn new_for_svg(size: Size, stream: OutputStream) -> Result<Self, Error> {
-        let surface = cairo::SvgSurface::for_stream(size.w, size.h, stream.into_write())?;
+        let mut surface = cairo::SvgSurface::for_stream(size.w, size.h, stream.into_write())?;
+        surface.set_document_unit(cairo::SvgUnit::User);
         Ok(Self::Svg(surface, size))
     }
 
+    #[cfg(not(system_deps_have_cairo_svg))]
+    fn new_for_svg(_size: Size, _stream: OutputStream) -> Result<Self, Error> {
+        Err(Error("unsupported format".to_string()))
+    }
+
     pub fn render(
         &self,
         renderer: &CairoRenderer,
@@ -271,8 +310,8 @@ impl Surface {
     }
 }
 
-fn checked_i32(x: f64) -> Result<i32, cairo::Status> {
-    cast::i32(x).map_err(|_| cairo::Status::InvalidSize)
+fn checked_i32(x: f64) -> Result<i32, cairo::Error> {
+    cast::i32(x).map_err(|_| cairo::Error::InvalidSize)
 }
 
 mod metadata {
@@ -298,33 +337,59 @@ mod metadata {
     }
 }
 
-// These Stdin and Stdout types can be removed once we depend on Rust 1.48
-
 struct Stdin;
 
 impl Stdin {
-    pub fn stream() -> UnixInputStream {
-        unsafe { UnixInputStream::new(Self {}) }
+    #[cfg(unix)]
+    pub fn stream() -> InputStream {
+        let stream = unsafe { UnixInputStream::new(0) };
+        stream.upcast::<InputStream>()
     }
-}
 
-impl std::os::unix::io::IntoRawFd for Stdin {
-    fn into_raw_fd(self) -> std::os::unix::io::RawFd {
-        0 as std::os::unix::io::RawFd
+    #[cfg(windows)]
+    pub fn stream() -> InputStream {
+        // https://github.com/gtk-rs/gtk-rs/issues/381 - do this with
+        // Win32InputStream::with_handle when this is fixed
+
+        extern "C" {
+            pub fn g_win32_input_stream_new(
+                handle: *mut c_void,
+                close_handle: gboolean,
+            ) -> *mut GInputStream;
+        }
+
+        let raw_handle = io::stdin().as_raw_handle();
+        unsafe {
+            InputStream::from_glib_full(g_win32_input_stream_new(raw_handle, false.to_glib()))
+        }
     }
 }
 
 struct Stdout;
 
 impl Stdout {
-    pub fn stream() -> UnixOutputStream {
-        unsafe { UnixOutputStream::new(Self {}) }
+    #[cfg(unix)]
+    pub fn stream() -> OutputStream {
+        let stream = unsafe { UnixOutputStream::new(1) };
+        stream.upcast::<OutputStream>()
     }
-}
 
-impl std::os::unix::io::IntoRawFd for Stdout {
-    fn into_raw_fd(self) -> std::os::unix::io::RawFd {
-        1 as std::os::unix::io::RawFd
+    #[cfg(windows)]
+    pub fn stream() -> OutputStream {
+        // https://github.com/gtk-rs/gtk-rs/issues/381 - do this with
+        // Win32OutputStream::with_handle when this is fixed
+
+        extern "C" {
+            pub fn g_win32_output_stream_new(
+                handle: *mut c_void,
+                close_handle: gboolean,
+            ) -> *mut GOutputStream;
+        }
+
+        let raw_handle = io::stdout().as_raw_handle();
+        unsafe {
+            OutputStream::from_glib_full(g_win32_output_stream_new(raw_handle, false.to_glib()))
+        }
     }
 }
 
@@ -400,7 +465,7 @@ impl Converter {
 
         for input in &self.input {
             let (stream, basefile) = match input {
-                Input::Stdin => (Stdin::stream().upcast::<InputStream>(), None),
+                Input::Stdin => (Stdin::stream(), None),
                 Input::Named(p) => {
                     let file = p.get_gfile();
                     let stream = file
@@ -502,7 +567,7 @@ impl Converter {
 
     fn create_surface(&self, size: Size) -> Result<Surface, Error> {
         let output_stream = match self.output {
-            Output::Stdout => Stdout::stream().upcast::<OutputStream>(),
+            Output::Stdout => Stdout::stream(),
             Output::Path(ref p) => {
                 let file = gio::File::new_for_path(p);
                 let stream = file
@@ -519,13 +584,13 @@ impl Converter {
 fn parse_args() -> Result<Converter, Error> {
     let supported_formats = vec![
         "Png",
-        #[cfg(have_cairo_pdf)]
+        #[cfg(system_deps_have_cairo_pdf)]
         "Pdf",
-        #[cfg(have_cairo_ps)]
+        #[cfg(system_deps_have_cairo_ps)]
         "Ps",
-        #[cfg(have_cairo_ps)]
+        #[cfg(system_deps_have_cairo_ps)]
         "Eps",
-        #[cfg(have_cairo_svg)]
+        #[cfg(system_deps_have_cairo_svg)]
         "Svg",
     ];
 
@@ -695,9 +760,10 @@ fn parse_args() -> Result<Converter, Error> {
 
     let input = match matches.values_of_os("FILE") {
         Some(values) => values
-            .map(PathOrUrl::from_os_str)
-            .map(Input::Named)
-            .collect(),
+            .map(|f| PathOrUrl::from_os_str(f).map_err(Error))
+            .map(|r| r.map(Input::Named))
+            .collect::<Result<Vec<Input>, Error>>()?,
+
         None => vec![Input::Stdin],
     };
 
