@@ -105,7 +105,6 @@ pub struct ElementInner<T: SetAttributes + Draw> {
     transform: Transform,
     values: ComputedValues,
     cond: bool,
-    style_attr: String,
     pub element_impl: T,
 }
 
@@ -129,12 +128,9 @@ impl<T: SetAttributes + Draw> ElementInner<T> {
             transform: Default::default(),
             values: Default::default(),
             cond: true,
-            style_attr: String::new(),
             element_impl,
         };
 
-        e.save_style_attribute();
-
         let mut set_attributes = || -> Result<(), ElementError> {
             e.set_transform_attribute()?;
             e.set_conditional_processing_attributes()?;
@@ -185,16 +181,6 @@ impl<T: SetAttributes + Draw> ElementInner<T> {
         self.transform
     }
 
-    fn save_style_attribute(&mut self) {
-        self.style_attr.push_str(
-            self.attributes
-                .iter()
-                .find(|(attr, _)| attr.expanded() == expanded_name!("", "style"))
-                .map(|(_, value)| value)
-                .unwrap_or(""),
-        );
-    }
-
     fn set_transform_attribute(&mut self) -> Result<(), ElementError> {
         self.transform = self
             .attributes
@@ -239,28 +225,10 @@ impl<T: SetAttributes + Draw> ElementInner<T> {
     }
 
     /// Hands the `attrs` to the node's state, to apply the presentation attributes.
+    #[allow(clippy::unnecessary_wraps)]
     fn set_presentation_attributes(&mut self) -> Result<(), ElementError> {
-        match self
-            .specified_values
+        self.specified_values
             .parse_presentation_attributes(&self.attributes)
-        {
-            Ok(_) => Ok(()),
-            Err(e) => {
-                // FIXME: we'll ignore errors here for now.
-                //
-                // If we set the node to be in error, we expose buggy handling of the
-                // enable-background property; we are not parsing it correctly. This
-                // causes tests/fixtures/reftests/bugs/587721-text-transform.svg to fail
-                // because it has enable-background="new 0 0 1179.75118 687.74173" in the
-                // toplevel svg element.
-                //
-                //   self.set_error(e);
-                //   return;
-
-                rsvg_log!("(attribute error: {})", e);
-                Ok(())
-            }
-        }
     }
 
     // Applies a style declaration to the node's specified_values
@@ -272,19 +240,22 @@ impl<T: SetAttributes + Draw> ElementInner<T> {
         );
     }
 
-    /// Applies CSS styles from the saved value of the "style" attribute
+    /// Applies CSS styles from the "style" attribute
     fn set_style_attribute(&mut self) {
-        if !self.style_attr.is_empty() {
+        let style = self
+            .attributes
+            .iter()
+            .find(|(attr, _)| attr.expanded() == expanded_name!("", "style"))
+            .map(|(_, value)| value);
+
+        if let Some(style) = style {
             if let Err(e) = self.specified_values.parse_style_declarations(
-                self.style_attr.as_str(),
+                style,
                 Origin::Author,
                 &mut self.important_styles,
             ) {
                 self.set_error(e);
             }
-
-            self.style_attr.clear();
-            self.style_attr.shrink_to_fit();
         }
     }
 
@@ -601,13 +572,13 @@ impl Element {
     pub fn is_accessed_by_reference(&self) -> bool {
         matches!(
             self,
-            Element::ClipPath(_) |
-            Element::Filter(_) |
-            Element::LinearGradient(_) |
-            Element::Marker(_) |
-            Element::Mask(_) |
-            Element::Pattern(_) |
-            Element::RadialGradient(_)
+            Element::ClipPath(_)
+                | Element::Filter(_)
+                | Element::LinearGradient(_)
+                | Element::Marker(_)
+                | Element::Mask(_)
+                | Element::Pattern(_)
+                | Element::RadialGradient(_)
         )
     }
 }
