@@ -1,16 +1,16 @@
 //! The `filter` element.
 
 use cssparser::Parser;
-use markup5ever::{expanded_name, local_name, namespace_url, ns, QualName};
+use markup5ever::{expanded_name, local_name, namespace_url, ns};
 use std::slice::Iter;
 
 use crate::coord_units::CoordUnits;
 use crate::document::{AcquiredNodes, NodeId};
 use crate::drawing_ctx::ViewParams;
 use crate::element::{Draw, Element, ElementResult, SetAttributes};
-use crate::error::{ElementError, ValueErrorKind};
+use crate::error::ValueErrorKind;
 use crate::length::*;
-use crate::node::{Node, NodeBorrow};
+use crate::node::NodeBorrow;
 use crate::parsers::{Parse, ParseValue};
 use crate::properties::ComputedValues;
 use crate::rect::Rect;
@@ -22,8 +22,15 @@ pub struct Filter {
     y: Length<Vertical>,
     width: ULength<Horizontal>,
     height: ULength<Vertical>,
-    filterunits: CoordUnits,
-    primitiveunits: CoordUnits,
+    filter_units: CoordUnits,
+    primitive_units: CoordUnits,
+}
+
+/// A <filter> element definition in user-space coordinates.
+pub struct ResolvedFilter {
+    pub rect: Rect,
+    pub filter_units: CoordUnits,
+    pub primitive_units: CoordUnits,
 }
 
 impl Default for Filter {
@@ -34,41 +41,34 @@ impl Default for Filter {
             y: Length::<Vertical>::parse_str("-10%").unwrap(),
             width: ULength::<Horizontal>::parse_str("120%").unwrap(),
             height: ULength::<Vertical>::parse_str("120%").unwrap(),
-            filterunits: CoordUnits::ObjectBoundingBox,
-            primitiveunits: CoordUnits::UserSpaceOnUse,
+            filter_units: CoordUnits::ObjectBoundingBox,
+            primitive_units: CoordUnits::UserSpaceOnUse,
         }
     }
 }
 
 impl Filter {
     pub fn get_filter_units(&self) -> CoordUnits {
-        self.filterunits
+        self.filter_units
     }
 
     pub fn get_primitive_units(&self) -> CoordUnits {
-        self.primitiveunits
+        self.primitive_units
     }
 
-    pub fn get_rect(&self, values: &ComputedValues, params: &ViewParams) -> Rect {
-        // With filterunits == ObjectBoundingBox, lengths represent fractions or percentages of the
-        // referencing node. No units are allowed (it's checked during attribute parsing).
-        let (x, y, w, h) = if self.filterunits == CoordUnits::ObjectBoundingBox {
-            (
-                self.x.length,
-                self.y.length,
-                self.width.length,
-                self.height.length,
-            )
-        } else {
-            (
-                self.x.normalize(values, &params),
-                self.y.normalize(values, &params),
-                self.width.normalize(values, &params),
-                self.height.normalize(values, &params),
-            )
-        };
+    pub fn resolve(&self, values: &ComputedValues, params: &ViewParams) -> ResolvedFilter {
+        let x = self.x.normalize(values, &params);
+        let y = self.y.normalize(values, &params);
+        let w = self.width.normalize(values, &params);
+        let h = self.height.normalize(values, &params);
 
-        Rect::new(x, y, x + w, y + h)
+        let rect = Rect::new(x, y, x + w, y + h);
+
+        ResolvedFilter {
+            rect,
+            filter_units: self.filter_units,
+            primitive_units: self.primitive_units,
+        }
     }
 }
 
@@ -80,36 +80,17 @@ impl SetAttributes for Filter {
             .find(|(attr, _)| attr.expanded() == expanded_name!("", "filterUnits"))
             .and_then(|(attr, value)| attr.parse(value).ok());
         if let Some(filter_units) = result {
-            self.filterunits = filter_units
+            self.filter_units = filter_units
         }
 
-        // With ObjectBoundingBox, only fractions and percents are allowed.
-        let units_allowed = self.filterunits != CoordUnits::ObjectBoundingBox;
-
         // Parse the rest of the attributes.
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "x") => {
-                    self.x = attr
-                        .parse(value)
-                        .and_then(|v| check_units(v, units_allowed, attr))?
-                }
-                expanded_name!("", "y") => {
-                    self.y = attr
-                        .parse(value)
-                        .and_then(|v| check_units(v, units_allowed, attr))?
-                }
-                expanded_name!("", "width") => {
-                    self.width = attr
-                        .parse(value)
-                        .and_then(|v| check_units(v, units_allowed, attr))?
-                }
-                expanded_name!("", "height") => {
-                    self.height = attr
-                        .parse(value)
-                        .and_then(|v| check_units(v, units_allowed, attr))?
-                }
-                expanded_name!("", "primitiveUnits") => self.primitiveunits = attr.parse(value)?,
+                expanded_name!("", "x") => self.x = attr.parse(value)?,
+                expanded_name!("", "y") => self.y = attr.parse(value)?,
+                expanded_name!("", "width") => self.width = attr.parse(value)?,
+                expanded_name!("", "height") => self.height = attr.parse(value)?,
+                expanded_name!("", "primitiveUnits") => self.primitive_units = attr.parse(value)?,
                 _ => (),
             }
         }
@@ -118,26 +99,6 @@ impl SetAttributes for Filter {
     }
 }
 
-fn check_units<N: Normalize, V: Validate>(
-    length: CssLength<N, V>,
-    units_allowed: bool,
-    attr: QualName,
-) -> Result<CssLength<N, V>, ElementError> {
-    if units_allowed {
-        return Ok(length);
-    }
-
-    match length.unit {
-        LengthUnit::Px | LengthUnit::Percent => Ok(length),
-        _ => Err(ElementError {
-            attr,
-            err: ValueErrorKind::parse_error(
-                "unit identifiers are not allowed with filterUnits set to objectBoundingBox",
-            ),
-        }),
-    }
-}
-
 impl Draw for Filter {}
 
 #[derive(Debug, Clone, PartialEq)]
@@ -166,7 +127,11 @@ impl FilterValueList {
 
     /// Check that at least one filter URI exists and that all contained
     /// URIs reference existing <filter> elements.
-    pub fn is_applicable(&self, node: &Node, acquired_nodes: &mut AcquiredNodes<'_>) -> bool {
+    ///
+    /// The `node_name` refers to the node being filtered; it is just
+    /// to log an error in case the filter value list is not
+    /// applicable.
+    pub fn is_applicable(&self, node_name: &str, acquired_nodes: &mut AcquiredNodes<'_>) -> bool {
         if self.is_empty() {
             return false;
         }
@@ -184,7 +149,7 @@ impl FilterValueList {
                         _ => {
                             rsvg_log!(
                                 "element {} will not be filtered since \"{}\" is not a filter",
-                                node,
+                                node_name,
                                 v,
                             );
                             false
@@ -194,7 +159,7 @@ impl FilterValueList {
                 _ => {
                     rsvg_log!(
                         "element {} will not be filtered since its filter \"{}\" was not found",
-                        node,
+                        node_name,
                         v,
                     );
                     false
