@@ -6,16 +6,19 @@ use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
 use crate::element::{ElementResult, SetAttributes};
 use crate::error::*;
-use crate::node::Node;
+use crate::node::{CascadedValues, Node};
 use crate::parsers::{NumberList, NumberListLength, Parse, ParseValue};
+use crate::property_defs::ColorInterpolationFilters;
+use crate::rect::IRect;
 use crate::surface_utils::{
     iterators::Pixels, shared_surface::ExclusiveImageSurface, ImageSurfaceDataExt, Pixel,
 };
 use crate::util::clamp;
 use crate::xml::Attributes;
 
-use super::context::{FilterContext, FilterOutput, FilterResult};
-use super::{FilterEffect, FilterError, PrimitiveWithInput};
+use super::bounds::BoundsBuilder;
+use super::context::{FilterContext, FilterOutput};
+use super::{FilterEffect, FilterError, Input, Primitive, PrimitiveParams};
 
 /// Color matrix operation types.
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
@@ -29,17 +32,27 @@ enum OperationType {
 enum_default!(OperationType, OperationType::Matrix);
 
 /// The `feColorMatrix` filter primitive.
+#[derive(Default)]
 pub struct FeColorMatrix {
-    base: PrimitiveWithInput,
-    matrix: Matrix5<f64>,
+    base: Primitive,
+    params: ColorMatrix,
 }
 
-impl Default for FeColorMatrix {
-    /// Constructs a new `ColorMatrix` with empty properties.
-    #[inline]
-    fn default() -> FeColorMatrix {
-        FeColorMatrix {
-            base: PrimitiveWithInput::new::<Self>(),
+/// Resolved `feColorMatrix` primitive for rendering.
+#[derive(Clone)]
+pub struct ColorMatrix {
+    in1: Input,
+    matrix: Matrix5<f64>,
+    color_interpolation_filters: ColorInterpolationFilters,
+}
+
+impl Default for ColorMatrix {
+    fn default() -> ColorMatrix {
+        ColorMatrix {
+            in1: Default::default(),
+            color_interpolation_filters: Default::default(),
+
+            // nalgebra's Default for Matrix5 is all zeroes, so we actually need this :(
             matrix: Matrix5::identity(),
         }
     }
@@ -48,7 +61,7 @@ impl Default for FeColorMatrix {
 #[rustfmt::skip]
 impl SetAttributes for FeColorMatrix {
     fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
-        self.base.set_attributes(attrs)?;
+        self.params.in1 = self.base.parse_one_input(attrs)?;
 
         // First, determine the operation type.
         let mut operation_type = Default::default();
@@ -62,7 +75,7 @@ impl SetAttributes for FeColorMatrix {
         // Now read the matrix correspondingly.
         // LuminanceToAlpha doesn't accept any matrix.
         if operation_type == OperationType::LuminanceToAlpha {
-            self.matrix = {
+            self.params.matrix = {
                 Matrix5::new(
                     0.0,    0.0,    0.0,    0.0, 0.0,
                     0.0,    0.0,    0.0,    0.0, 0.0,
@@ -129,7 +142,7 @@ impl SetAttributes for FeColorMatrix {
                     }
                 };
 
-                self.matrix = new_matrix;
+                self.params.matrix = new_matrix;
             }
         }
 
@@ -137,29 +150,34 @@ impl SetAttributes for FeColorMatrix {
     }
 }
 
-impl FilterEffect for FeColorMatrix {
-    fn render(
+impl ColorMatrix {
+    pub fn render(
         &self,
-        node: &Node,
+        bounds_builder: BoundsBuilder,
         ctx: &FilterContext,
         acquired_nodes: &mut AcquiredNodes<'_>,
         draw_ctx: &mut DrawingCtx,
-    ) -> Result<FilterResult, FilterError> {
-        let input = self.base.get_input(ctx, acquired_nodes, draw_ctx)?;
-        let bounds = self
-            .base
-            .get_bounds(ctx, node.parent().as_ref())?
-            .add_input(&input)
-            .into_irect(ctx, draw_ctx);
+    ) -> Result<FilterOutput, FilterError> {
+        let input_1 = ctx.get_input(
+            acquired_nodes,
+            draw_ctx,
+            &self.in1,
+            self.color_interpolation_filters,
+        )?;
+        let bounds: IRect = bounds_builder
+            .add_input(&input_1)
+            .compute(ctx)
+            .clipped
+            .into();
 
         let mut surface = ExclusiveImageSurface::new(
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-            input.surface().surface_type(),
+            input_1.surface().surface_type(),
         )?;
 
         surface.modify(&mut |data, stride| {
-            for (x, y, pixel) in Pixels::within(input.surface(), bounds) {
+            for (x, y, pixel) in Pixels::within(input_1.surface(), bounds) {
                 let alpha = f64::from(pixel.a) / 255f64;
 
                 let pixel_vec = if alpha == 0.0 {
@@ -191,18 +209,22 @@ impl FilterEffect for FeColorMatrix {
             }
         });
 
-        Ok(FilterResult {
-            name: self.base.result.clone(),
-            output: FilterOutput {
-                surface: surface.share()?,
-                bounds,
-            },
+        Ok(FilterOutput {
+            surface: surface.share()?,
+            bounds,
         })
     }
+}
 
-    #[inline]
-    fn is_affected_by_color_interpolation_filters(&self) -> bool {
-        true
+impl FilterEffect for FeColorMatrix {
+    fn resolve(&self, node: &Node) -> Result<(Primitive, PrimitiveParams), FilterError> {
+        let cascaded = CascadedValues::new_from_node(node);
+        let values = cascaded.get();
+
+        let mut params = self.params.clone();
+        params.color_interpolation_filters = values.color_interpolation_filters();
+
+        Ok((self.base.clone(), PrimitiveParams::ColorMatrix(params)))
     }
 }
 
