@@ -7,35 +7,39 @@ use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
 use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
-use crate::node::{Node, NodeBorrow};
+use crate::node::{CascadedValues, Node, NodeBorrow};
 use crate::parsers::{NumberList, NumberListLength, Parse, ParseValue};
+use crate::property_defs::ColorInterpolationFilters;
+use crate::rect::IRect;
 use crate::surface_utils::{
     iterators::Pixels, shared_surface::ExclusiveImageSurface, ImageSurfaceDataExt, Pixel,
 };
 use crate::util::clamp;
 use crate::xml::Attributes;
 
-use super::context::{FilterContext, FilterOutput, FilterResult};
-use super::{FilterEffect, FilterError, PrimitiveWithInput};
+use super::bounds::BoundsBuilder;
+use super::context::{FilterContext, FilterOutput};
+use super::{FilterEffect, FilterError, Input, Primitive, PrimitiveParams};
 
 /// The `feComponentTransfer` filter primitive.
+#[derive(Default)]
 pub struct FeComponentTransfer {
-    base: PrimitiveWithInput,
+    base: Primitive,
+    params: ComponentTransfer,
 }
 
-impl Default for FeComponentTransfer {
-    /// Constructs a new `ComponentTransfer` with empty properties.
-    #[inline]
-    fn default() -> FeComponentTransfer {
-        FeComponentTransfer {
-            base: PrimitiveWithInput::new::<Self>(),
-        }
-    }
+/// Resolved `feComponentTransfer` primitive for rendering.
+#[derive(Clone, Default)]
+pub struct ComponentTransfer {
+    in1: Input,
+    functions: Functions,
+    color_interpolation_filters: ColorInterpolationFilters,
 }
 
 impl SetAttributes for FeComponentTransfer {
     fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
-        self.base.set_attributes(attrs)
+        self.params.in1 = self.base.parse_one_input(attrs)?;
+        Ok(())
     }
 }
 
@@ -49,6 +53,7 @@ enum Channel {
 }
 
 /// Component transfer function types.
+#[derive(Clone, Debug, PartialEq)]
 enum FunctionType {
     Identity,
     Table,
@@ -71,8 +76,8 @@ impl Parse for FunctionType {
 }
 
 /// The compute function parameters.
-struct FunctionParameters<'a> {
-    table_values: &'a Vec<f64>,
+struct FunctionParameters {
+    table_values: Vec<f64>,
     slope: f64,
     intercept: f64,
     amplitude: f64,
@@ -80,16 +85,24 @@ struct FunctionParameters<'a> {
     offset: f64,
 }
 
+#[derive(Clone, Debug, Default, PartialEq)]
+struct Functions {
+    r: FeFuncR,
+    g: FeFuncG,
+    b: FeFuncB,
+    a: FeFuncA,
+}
+
 /// The compute function type.
-type Function = fn(&FunctionParameters<'_>, f64) -> f64;
+type Function = fn(&FunctionParameters, f64) -> f64;
 
 /// The identity component transfer function.
-fn identity(_: &FunctionParameters<'_>, value: f64) -> f64 {
+fn identity(_: &FunctionParameters, value: f64) -> f64 {
     value
 }
 
 /// The table component transfer function.
-fn table(params: &FunctionParameters<'_>, value: f64) -> f64 {
+fn table(params: &FunctionParameters, value: f64) -> f64 {
     let n = params.table_values.len() - 1;
     let k = (value * (n as f64)).floor() as usize;
 
@@ -108,7 +121,7 @@ fn table(params: &FunctionParameters<'_>, value: f64) -> f64 {
 }
 
 /// The discrete component transfer function.
-fn discrete(params: &FunctionParameters<'_>, value: f64) -> f64 {
+fn discrete(params: &FunctionParameters, value: f64) -> f64 {
     let n = params.table_values.len();
     let k = (value * (n as f64)).floor() as usize;
 
@@ -116,12 +129,12 @@ fn discrete(params: &FunctionParameters<'_>, value: f64) -> f64 {
 }
 
 /// The linear component transfer function.
-fn linear(params: &FunctionParameters<'_>, value: f64) -> f64 {
+fn linear(params: &FunctionParameters, value: f64) -> f64 {
     params.slope * value + params.intercept
 }
 
 /// The gamma component transfer function.
-fn gamma(params: &FunctionParameters<'_>, value: f64) -> f64 {
+fn gamma(params: &FunctionParameters, value: f64) -> f64 {
     params.amplitude * value.powf(params.exponent) + params.offset
 }
 
@@ -130,7 +143,7 @@ trait FeComponentTransferFunc {
     fn function(&self) -> Function;
 
     /// Returns the component transfer function parameters.
-    fn function_parameters(&self) -> FunctionParameters<'_>;
+    fn function_parameters(&self) -> FunctionParameters;
 
     /// Returns the channel.
     fn channel(&self) -> Channel;
@@ -138,6 +151,7 @@ trait FeComponentTransferFunc {
 
 macro_rules! func_x {
     ($func_name:ident, $channel:expr) => {
+        #[derive(Clone, Debug, PartialEq)]
         pub struct $func_name {
             channel: Channel,
             function_type: FunctionType,
@@ -167,9 +181,9 @@ macro_rules! func_x {
 
         impl FeComponentTransferFunc for $func_name {
             #[inline]
-            fn function_parameters(&self) -> FunctionParameters<'_> {
+            fn function_parameters(&self) -> FunctionParameters {
                 FunctionParameters {
-                    table_values: &self.table_values,
+                    table_values: self.table_values.clone(),
                     slope: self.slope,
                     intercept: self.intercept,
                     amplitude: self.amplitude,
@@ -249,16 +263,13 @@ func_x!(FeFuncB, Channel::B);
 func_x!(FeFuncA, Channel::A);
 
 macro_rules! func_or_default {
-    ($func_node:ident, $func_type:ident, $func_data:ident, $func_default:ident) => {
+    ($func_node:ident, $func_type:ident) => {
         match $func_node {
-            Some(ref f) => {
-                $func_data = f.borrow_element();
-                match *$func_data {
-                    Element::$func_type(ref e) => &*e,
-                    _ => unreachable!(),
-                }
-            }
-            _ => &$func_default,
+            Some(ref f) => match *f.borrow_element() {
+                Element::$func_type(ref e) => e.element_impl.clone(),
+                _ => unreachable!(),
+            },
+            _ => $func_type::default(),
         };
     };
 }
@@ -276,61 +287,35 @@ macro_rules! get_func_x_node {
     };
 }
 
-impl FilterEffect for FeComponentTransfer {
-    fn render(
+impl ComponentTransfer {
+    pub fn render(
         &self,
-        node: &Node,
+        bounds_builder: BoundsBuilder,
         ctx: &FilterContext,
         acquired_nodes: &mut AcquiredNodes<'_>,
         draw_ctx: &mut DrawingCtx,
-    ) -> Result<FilterResult, FilterError> {
-        let input = self.base.get_input(ctx, acquired_nodes, draw_ctx)?;
-        let bounds = self
-            .base
-            .get_bounds(ctx, node.parent().as_ref())?
-            .add_input(&input)
-            .into_irect(ctx, draw_ctx);
+    ) -> Result<FilterOutput, FilterError> {
+        let input_1 = ctx.get_input(
+            acquired_nodes,
+            draw_ctx,
+            &self.in1,
+            self.color_interpolation_filters,
+        )?;
+        let bounds: IRect = bounds_builder
+            .add_input(&input_1)
+            .compute(ctx)
+            .clipped
+            .into();
 
         // Create the output surface.
         let mut surface = ExclusiveImageSurface::new(
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-            input.surface().surface_type(),
+            input_1.surface().surface_type(),
         )?;
 
-        let func_r_node = get_func_x_node!(node, FeFuncR, Channel::R);
-        let func_g_node = get_func_x_node!(node, FeFuncG, Channel::G);
-        let func_b_node = get_func_x_node!(node, FeFuncB, Channel::B);
-        let func_a_node = get_func_x_node!(node, FeFuncA, Channel::A);
-
-        for node in [&func_r_node, &func_g_node, &func_b_node, &func_a_node]
-            .iter()
-            .filter_map(|x| x.as_ref())
-        {
-            if node.borrow_element().is_in_error() {
-                return Err(FilterError::ChildNodeInError);
-            }
-        }
-
-        // These are the default funcs that perform an identity transformation.
-        let func_r_default = FeFuncR::default();
-        let func_g_default = FeFuncG::default();
-        let func_b_default = FeFuncB::default();
-        let func_a_default = FeFuncA::default();
-
-        // We need to tell the borrow checker that these live long enough
-        let func_r_element;
-        let func_g_element;
-        let func_b_element;
-        let func_a_element;
-
-        let func_r = func_or_default!(func_r_node, FeFuncR, func_r_element, func_r_default);
-        let func_g = func_or_default!(func_g_node, FeFuncG, func_g_element, func_g_default);
-        let func_b = func_or_default!(func_b_node, FeFuncB, func_b_element, func_b_default);
-        let func_a = func_or_default!(func_a_node, FeFuncA, func_a_element, func_a_default);
-
         #[inline]
-        fn compute_func<F>(func: &F) -> impl Fn(u8, f64, f64) -> u8 + '_
+        fn compute_func<F>(func: &F) -> impl Fn(u8, f64, f64) -> u8
         where
             F: FeComponentTransferFunc,
         {
@@ -349,18 +334,18 @@ impl FilterEffect for FeComponentTransfer {
             }
         }
 
-        let compute_r = compute_func::<FeFuncR>(&func_r);
-        let compute_g = compute_func::<FeFuncG>(&func_g);
-        let compute_b = compute_func::<FeFuncB>(&func_b);
+        let compute_r = compute_func::<FeFuncR>(&self.functions.r);
+        let compute_g = compute_func::<FeFuncG>(&self.functions.g);
+        let compute_b = compute_func::<FeFuncB>(&self.functions.b);
 
         // Alpha gets special handling since everything else depends on it.
-        let compute_a = func_a.function();
-        let params_a = func_a.function_parameters();
+        let compute_a = self.functions.a.function();
+        let params_a = self.functions.a.function_parameters();
         let compute_a = |alpha| compute_a(&params_a, alpha);
 
         // Do the actual processing.
         surface.modify(&mut |data, stride| {
-            for (x, y, pixel) in Pixels::within(input.surface(), bounds) {
+            for (x, y, pixel) in Pixels::within(input_1.surface(), bounds) {
                 let alpha = f64::from(pixel.a) / 255f64;
                 let new_alpha = compute_a(alpha);
 
@@ -375,16 +360,107 @@ impl FilterEffect for FeComponentTransfer {
             }
         });
 
-        Ok(FilterResult {
-            name: self.base.result.clone(),
-            output: FilterOutput {
-                surface: surface.share()?,
-                bounds,
-            },
+        Ok(FilterOutput {
+            surface: surface.share()?,
+            bounds,
         })
     }
+}
 
-    fn is_affected_by_color_interpolation_filters(&self) -> bool {
-        true
+impl FilterEffect for FeComponentTransfer {
+    fn resolve(&self, node: &Node) -> Result<(Primitive, PrimitiveParams), FilterError> {
+        let cascaded = CascadedValues::new_from_node(node);
+        let values = cascaded.get();
+
+        let mut params = self.params.clone();
+        params.functions = get_functions(node)?;
+        params.color_interpolation_filters = values.color_interpolation_filters();
+
+        Ok((
+            self.base.clone(),
+            PrimitiveParams::ComponentTransfer(params),
+        ))
+    }
+}
+
+/// Takes a feComponentTransfer and walks its children to produce the feFuncX arguments.
+fn get_functions(node: &Node) -> Result<Functions, FilterError> {
+    let func_r_node = get_func_x_node!(node, FeFuncR, Channel::R);
+    let func_g_node = get_func_x_node!(node, FeFuncG, Channel::G);
+    let func_b_node = get_func_x_node!(node, FeFuncB, Channel::B);
+    let func_a_node = get_func_x_node!(node, FeFuncA, Channel::A);
+
+    for node in [&func_r_node, &func_g_node, &func_b_node, &func_a_node]
+        .iter()
+        .filter_map(|x| x.as_ref())
+    {
+        if node.borrow_element().is_in_error() {
+            return Err(FilterError::ChildNodeInError);
+        }
+    }
+
+    let r = func_or_default!(func_r_node, FeFuncR);
+    let g = func_or_default!(func_g_node, FeFuncG);
+    let b = func_or_default!(func_b_node, FeFuncB);
+    let a = func_or_default!(func_a_node, FeFuncA);
+
+    Ok(Functions { r, g, b, a })
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::document::Document;
+
+    #[test]
+    fn extracts_functions() {
+        let document = Document::load_from_bytes(
+            br#"<?xml version="1.0" encoding="UTF-8"?>
+<svg xmlns="http://www.w3.org/2000/svg">
+  <filter id="filter">
+    <feComponentTransfer id="component_transfer">
+      <!-- no feFuncR so it should get the defaults -->
+
+      <feFuncG type="table" tableValues="0.0 1.0 2.0"/>
+
+      <feFuncB type="table"/>
+      <!-- duplicate this to test that last-one-wins -->
+      <feFuncB type="discrete" tableValues="0.0, 1.0" slope="1.0" intercept="2.0" amplitude="3.0" exponent="4.0" offset="5.0"/>
+
+      <!-- no feFuncA so it should get the defaults -->
+    </feComponentTransfer>
+  </filter>
+</svg>
+"#
+        );
+
+        let component_transfer = document.lookup_internal_node("component_transfer").unwrap();
+        let functions = get_functions(&component_transfer).unwrap();
+
+        assert_eq!(
+            functions,
+            Functions {
+                r: FeFuncR::default(),
+
+                g: FeFuncG {
+                    function_type: FunctionType::Table,
+                    table_values: vec![0.0, 1.0, 2.0],
+                    ..FeFuncG::default()
+                },
+
+                b: FeFuncB {
+                    function_type: FunctionType::Discrete,
+                    table_values: vec![0.0, 1.0],
+                    slope: 1.0,
+                    intercept: 2.0,
+                    amplitude: 3.0,
+                    exponent: 4.0,
+                    offset: 5.0,
+                    ..FeFuncB::default()
+                },
+
+                a: FeFuncA::default(),
+            }
+        );
     }
 }
