@@ -5,12 +5,15 @@ use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
 use crate::element::{ElementResult, SetAttributes};
 use crate::error::*;
-use crate::node::Node;
+use crate::node::{CascadedValues, Node};
 use crate::parsers::{Parse, ParseValue};
+use crate::property_defs::ColorInterpolationFilters;
+use crate::rect::IRect;
 use crate::xml::Attributes;
 
-use super::context::{FilterContext, FilterOutput, FilterResult};
-use super::{FilterEffect, FilterError, Input, PrimitiveWithInput};
+use super::bounds::BoundsBuilder;
+use super::context::{FilterContext, FilterOutput};
+use super::{FilterEffect, FilterError, Input, Primitive, PrimitiveParams};
 
 /// Enumeration of the possible compositing operations.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
@@ -23,45 +26,41 @@ enum Operator {
     Arithmetic,
 }
 
+enum_default!(Operator, Operator::Over);
+
 /// The `feComposite` filter primitive.
+#[derive(Default)]
 pub struct FeComposite {
-    base: PrimitiveWithInput,
-    in2: Option<Input>,
+    base: Primitive,
+    params: Composite,
+}
+
+/// Resolved `feComposite` primitive for rendering.
+#[derive(Clone, Default)]
+pub struct Composite {
+    in1: Input,
+    in2: Input,
     operator: Operator,
     k1: f64,
     k2: f64,
     k3: f64,
     k4: f64,
-}
-
-impl Default for FeComposite {
-    /// Constructs a new `Composite` with empty properties.
-    #[inline]
-    fn default() -> FeComposite {
-        FeComposite {
-            base: PrimitiveWithInput::new::<Self>(),
-            in2: None,
-            operator: Operator::Over,
-            k1: 0.0,
-            k2: 0.0,
-            k3: 0.0,
-            k4: 0.0,
-        }
-    }
+    color_interpolation_filters: ColorInterpolationFilters,
 }
 
 impl SetAttributes for FeComposite {
     fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
-        self.base.set_attributes(attrs)?;
+        let (in1, in2) = self.base.parse_two_inputs(attrs)?;
+        self.params.in1 = in1;
+        self.params.in2 = in2;
 
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "in2") => self.in2 = attr.parse(value)?,
-                expanded_name!("", "operator") => self.operator = attr.parse(value)?,
-                expanded_name!("", "k1") => self.k1 = attr.parse(value)?,
-                expanded_name!("", "k2") => self.k2 = attr.parse(value)?,
-                expanded_name!("", "k3") => self.k3 = attr.parse(value)?,
-                expanded_name!("", "k4") => self.k4 = attr.parse(value)?,
+                expanded_name!("", "operator") => self.params.operator = attr.parse(value)?,
+                expanded_name!("", "k1") => self.params.k1 = attr.parse(value)?,
+                expanded_name!("", "k2") => self.params.k2 = attr.parse(value)?,
+                expanded_name!("", "k3") => self.params.k3 = attr.parse(value)?,
+                expanded_name!("", "k4") => self.params.k4 = attr.parse(value)?,
                 _ => (),
             }
         }
@@ -70,25 +69,35 @@ impl SetAttributes for FeComposite {
     }
 }
 
-impl FilterEffect for FeComposite {
-    fn render(
+impl Composite {
+    pub fn render(
         &self,
-        node: &Node,
+        bounds_builder: BoundsBuilder,
         ctx: &FilterContext,
         acquired_nodes: &mut AcquiredNodes<'_>,
         draw_ctx: &mut DrawingCtx,
-    ) -> Result<FilterResult, FilterError> {
-        let input = self.base.get_input(ctx, acquired_nodes, draw_ctx)?;
-        let input_2 = ctx.get_input(acquired_nodes, draw_ctx, self.in2.as_ref())?;
-        let bounds = self
-            .base
-            .get_bounds(ctx, node.parent().as_ref())?
-            .add_input(&input)
+    ) -> Result<FilterOutput, FilterError> {
+        let input_1 = ctx.get_input(
+            acquired_nodes,
+            draw_ctx,
+            &self.in1,
+            self.color_interpolation_filters,
+        )?;
+        let input_2 = ctx.get_input(
+            acquired_nodes,
+            draw_ctx,
+            &self.in2,
+            self.color_interpolation_filters,
+        )?;
+        let bounds: IRect = bounds_builder
+            .add_input(&input_1)
             .add_input(&input_2)
-            .into_irect(ctx, draw_ctx);
+            .compute(ctx)
+            .clipped
+            .into();
 
         let surface = if self.operator == Operator::Arithmetic {
-            input.surface().compose_arithmetic(
+            input_1.surface().compose_arithmetic(
                 input_2.surface(),
                 bounds,
                 self.k1,
@@ -97,22 +106,26 @@ impl FilterEffect for FeComposite {
                 self.k4,
             )?
         } else {
-            input.surface().compose(
+            input_1.surface().compose(
                 input_2.surface(),
                 bounds,
                 cairo::Operator::from(self.operator),
             )?
         };
 
-        Ok(FilterResult {
-            name: self.base.result.clone(),
-            output: FilterOutput { surface, bounds },
-        })
+        Ok(FilterOutput { surface, bounds })
     }
+}
 
-    #[inline]
-    fn is_affected_by_color_interpolation_filters(&self) -> bool {
-        true
+impl FilterEffect for FeComposite {
+    fn resolve(&self, node: &Node) -> Result<(Primitive, PrimitiveParams), FilterError> {
+        let cascaded = CascadedValues::new_from_node(node);
+        let values = cascaded.get();
+
+        let mut params = self.params.clone();
+        params.color_interpolation_filters = values.color_interpolation_filters();
+
+        Ok((self.base.clone(), PrimitiveParams::Composite(params)))
     }
 }
 
