@@ -11,10 +11,12 @@ use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
 use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::filters::{
-    context::{FilterContext, FilterOutput, FilterResult},
-    FilterEffect, FilterError, PrimitiveWithInput,
+    bounds::BoundsBuilder,
+    context::{FilterContext, FilterOutput},
+    FilterEffect, FilterError, Input, Primitive, PrimitiveParams,
 };
 use crate::node::{CascadedValues, Node, NodeBorrow};
+use crate::paint_server::resolve_color;
 use crate::parsers::{NonNegative, NumberOptionalNumber, ParseValue};
 use crate::property_defs::ColorInterpolationFilters;
 use crate::rect::IRect;
@@ -23,11 +25,86 @@ use crate::surface_utils::{
     ImageSurfaceDataExt, Pixel,
 };
 use crate::transform::Transform;
+use crate::unit_interval::UnitInterval;
 use crate::util::clamp;
 use crate::xml::Attributes;
 
+/// The `feDiffuseLighting` filter primitives.
+#[derive(Default)]
+pub struct FeDiffuseLighting {
+    base: Primitive,
+    params: DiffuseLightingParams,
+}
+
+#[derive(Clone)]
+pub struct DiffuseLightingParams {
+    in1: Input,
+    surface_scale: f64,
+    kernel_unit_length: Option<(f64, f64)>,
+    diffuse_constant: f64,
+}
+
+impl Default for DiffuseLightingParams {
+    fn default() -> Self {
+        Self {
+            in1: Default::default(),
+            surface_scale: 1.0,
+            kernel_unit_length: None,
+            diffuse_constant: 1.0,
+        }
+    }
+}
+
+/// The `feSpecularLighting` filter primitives.
+#[derive(Default)]
+pub struct FeSpecularLighting {
+    base: Primitive,
+    params: SpecularLightingParams,
+}
+
+#[derive(Clone)]
+pub struct SpecularLightingParams {
+    in1: Input,
+    surface_scale: f64,
+    kernel_unit_length: Option<(f64, f64)>,
+    specular_constant: f64,
+    specular_exponent: f64,
+}
+
+impl Default for SpecularLightingParams {
+    fn default() -> Self {
+        Self {
+            in1: Default::default(),
+            surface_scale: 1.0,
+            kernel_unit_length: None,
+            specular_constant: 1.0,
+            specular_exponent: 1.0,
+        }
+    }
+}
+
+/// Resolved `feDiffuseLighting` primitive for rendering.
+pub struct DiffuseLighting {
+    params: DiffuseLightingParams,
+    light: Light,
+}
+
+/// Resolved `feSpecularLighting` primitive for rendering.
+pub struct SpecularLighting {
+    params: SpecularLightingParams,
+    light: Light,
+}
+
+/// A light source before applying affine transformations, straight out of the SVG.
+#[derive(Debug, PartialEq)]
+enum UntransformedLightSource {
+    Distant(FeDistantLight),
+    Point(FePointLight),
+    Spot(FeSpotLight),
+}
+
 /// A light source with affine transformations applied.
-pub enum LightSource {
+enum LightSource {
     Distant {
         azimuth: f64,
         elevation: f64,
@@ -43,56 +120,33 @@ pub enum LightSource {
     },
 }
 
+impl UntransformedLightSource {
+    fn transform(&self, paffine: Transform) -> LightSource {
+        match *self {
+            UntransformedLightSource::Distant(ref l) => l.transform(),
+            UntransformedLightSource::Point(ref l) => l.transform(paffine),
+            UntransformedLightSource::Spot(ref l) => l.transform(paffine),
+        }
+    }
+}
+
 struct Light {
-    source: LightSource,
+    source: UntransformedLightSource,
     lighting_color: cssparser::RGBA,
     color_interpolation_filters: ColorInterpolationFilters,
 }
 
 impl Light {
-    pub fn new(node: &Node, paffine: Transform) -> Result<Light, FilterError> {
-        let mut sources = node.children().rev().filter(|c| {
-            c.is_element() && matches!(*c.borrow_element(), Element::FeDistantLight(_) | Element::FePointLight(_) | Element::FeSpotLight(_))
-        });
-
-        let source_node = sources.next();
-        if source_node.is_none() || sources.next().is_some() {
-            return Err(FilterError::InvalidLightSourceCount);
-        }
-
-        let source_node = source_node.unwrap();
-        let elt = source_node.borrow_element();
-
-        if elt.is_in_error() {
-            return Err(FilterError::ChildNodeInError);
-        }
-
-        let source = match *elt {
-            Element::FeDistantLight(ref l) => l.transform(),
-            Element::FePointLight(ref l) => l.transform(paffine),
-            Element::FeSpotLight(ref l) => l.transform(paffine),
-            _ => unreachable!(),
-        };
-
-        let cascaded = CascadedValues::new_from_node(node);
-        let values = cascaded.get();
-
-        let lighting_color = match values.lighting_color().0 {
-            cssparser::Color::CurrentColor => values.color().0,
-            cssparser::Color::RGBA(rgba) => rgba,
-        };
-
-        Ok(Light {
-            source,
-            lighting_color,
-            color_interpolation_filters: values.color_interpolation_filters(),
-        })
-    }
-
     /// Returns the color and unit (or null) vector from the image sample to the light.
     #[inline]
-    pub fn color_and_vector(&self, x: f64, y: f64, z: f64) -> (cssparser::RGBA, Vector3<f64>) {
-        let vector = match self.source {
+    pub fn color_and_vector(
+        &self,
+        source: &LightSource,
+        x: f64,
+        y: f64,
+        z: f64,
+    ) -> (cssparser::RGBA, Vector3<f64>) {
+        let vector = match *source {
             LightSource::Distant { azimuth, elevation } => {
                 let azimuth = azimuth.to_radians();
                 let elevation = elevation.to_radians();
@@ -109,7 +163,7 @@ impl Light {
             }
         };
 
-        let color = match self.source {
+        let color = match *source {
             LightSource::Spot {
                 direction,
                 specular_exponent,
@@ -142,14 +196,14 @@ impl Light {
     }
 }
 
-#[derive(Default)]
+#[derive(Clone, Debug, Default, PartialEq)]
 pub struct FeDistantLight {
     azimuth: f64,
     elevation: f64,
 }
 
 impl FeDistantLight {
-    pub fn transform(&self) -> LightSource {
+    fn transform(&self) -> LightSource {
         LightSource::Distant {
             azimuth: self.azimuth,
             elevation: self.elevation,
@@ -173,7 +227,7 @@ impl SetAttributes for FeDistantLight {
 
 impl Draw for FeDistantLight {}
 
-#[derive(Default)]
+#[derive(Clone, Debug, Default, PartialEq)]
 pub struct FePointLight {
     x: f64,
     y: f64,
@@ -181,7 +235,7 @@ pub struct FePointLight {
 }
 
 impl FePointLight {
-    pub fn transform(&self, paffine: Transform) -> LightSource {
+    fn transform(&self, paffine: Transform) -> LightSource {
         let (x, y) = paffine.transform_point(self.x, self.y);
         let z = transform_dist(paffine, self.z);
 
@@ -208,7 +262,7 @@ impl SetAttributes for FePointLight {
 
 impl Draw for FePointLight {}
 
-#[derive(Default)]
+#[derive(Clone, Debug, Default, PartialEq)]
 pub struct FeSpotLight {
     x: f64,
     y: f64,
@@ -221,7 +275,7 @@ pub struct FeSpotLight {
 }
 
 impl FeSpotLight {
-    pub fn transform(&self, paffine: Transform) -> LightSource {
+    fn transform(&self, paffine: Transform) -> LightSource {
         let (x, y) = paffine.transform_point(self.x, self.y);
         let z = transform_dist(paffine, self.z);
         let (points_at_x, points_at_y) =
@@ -276,39 +330,22 @@ fn transform_dist(t: Transform, d: f64) -> f64 {
     d * (t.xx.powi(2) + t.yy.powi(2)).sqrt() / std::f64::consts::SQRT_2
 }
 
-/// The `feDiffuseLighting` filter primitives.
-pub struct FeDiffuseLighting {
-    base: PrimitiveWithInput,
-    surface_scale: f64,
-    kernel_unit_length: Option<(f64, f64)>,
-    diffuse_constant: f64,
-}
-
-impl Default for FeDiffuseLighting {
-    fn default() -> Self {
-        Self {
-            base: PrimitiveWithInput::new::<Self>(),
-            surface_scale: 1.0,
-            kernel_unit_length: None,
-            diffuse_constant: 1.0,
-        }
-    }
-}
-
 impl SetAttributes for FeDiffuseLighting {
     fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
-        self.base.set_attributes(attrs)?;
+        self.params.in1 = self.base.parse_one_input(attrs)?;
 
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "surfaceScale") => self.surface_scale = attr.parse(value)?,
+                expanded_name!("", "surfaceScale") => {
+                    self.params.surface_scale = attr.parse(value)?
+                }
                 expanded_name!("", "kernelUnitLength") => {
                     let NumberOptionalNumber(NonNegative(x), NonNegative(y)) = attr.parse(value)?;
-                    self.kernel_unit_length = Some((x, y));
+                    self.params.kernel_unit_length = Some((x, y));
                 }
                 expanded_name!("", "diffuseConstant") => {
                     let NonNegative(c) = attr.parse(value)?;
-                    self.diffuse_constant = c;
+                    self.params.diffuse_constant = c;
                 }
                 _ => (),
             }
@@ -318,7 +355,7 @@ impl SetAttributes for FeDiffuseLighting {
     }
 }
 
-impl FeDiffuseLighting {
+impl DiffuseLighting {
     #[inline]
     fn compute_factor(&self, normal: Normal, light_vector: Vector3<f64>) -> f64 {
         let k = if normal.normal.is_zero() {
@@ -327,55 +364,36 @@ impl FeDiffuseLighting {
         } else {
             let mut n = normal
                 .normal
-                .map(|x| f64::from(x) * self.surface_scale / 255.);
+                .map(|x| f64::from(x) * self.params.surface_scale / 255.);
             n.component_mul_assign(&normal.factor);
             let normal = Vector3::new(n.x, n.y, 1.0);
 
             normal.dot(&light_vector) / normal.norm()
         };
 
-        self.diffuse_constant * k
-    }
-}
-
-/// The `feSpecularLighting` filter primitives.
-pub struct FeSpecularLighting {
-    base: PrimitiveWithInput,
-    surface_scale: f64,
-    kernel_unit_length: Option<(f64, f64)>,
-    specular_constant: f64,
-    specular_exponent: f64,
-}
-
-impl Default for FeSpecularLighting {
-    fn default() -> Self {
-        Self {
-            base: PrimitiveWithInput::new::<Self>(),
-            surface_scale: 1.0,
-            kernel_unit_length: None,
-            specular_constant: 1.0,
-            specular_exponent: 1.0,
-        }
+        self.params.diffuse_constant * k
     }
 }
 
 impl SetAttributes for FeSpecularLighting {
     fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
-        self.base.set_attributes(attrs)?;
+        self.params.in1 = self.base.parse_one_input(attrs)?;
 
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "surfaceScale") => self.surface_scale = attr.parse(value)?,
+                expanded_name!("", "surfaceScale") => {
+                    self.params.surface_scale = attr.parse(value)?
+                }
                 expanded_name!("", "kernelUnitLength") => {
                     let NumberOptionalNumber(NonNegative(x), NonNegative(y)) = attr.parse(value)?;
-                    self.kernel_unit_length = Some((x, y));
+                    self.params.kernel_unit_length = Some((x, y));
                 }
                 expanded_name!("", "specularConstant") => {
                     let NonNegative(c) = attr.parse(value)?;
-                    self.specular_constant = c;
+                    self.params.specular_constant = c;
                 }
                 expanded_name!("", "specularExponent") => {
-                    self.specular_exponent = attr.parse(value)?;
+                    self.params.specular_exponent = attr.parse(value)?;
                 }
                 _ => (),
             }
@@ -385,7 +403,7 @@ impl SetAttributes for FeSpecularLighting {
     }
 }
 
-impl FeSpecularLighting {
+impl SpecularLighting {
     #[inline]
     fn compute_factor(&self, normal: Normal, light_vector: Vector3<f64>) -> f64 {
         let h = light_vector + Vector3::new(0.0, 0.0, 1.0);
@@ -401,45 +419,49 @@ impl FeSpecularLighting {
         } else {
             let mut n = normal
                 .normal
-                .map(|x| f64::from(x) * self.surface_scale / 255.);
+                .map(|x| f64::from(x) * self.params.surface_scale / 255.);
             n.component_mul_assign(&normal.factor);
             let normal = Vector3::new(n.x, n.y, 1.0);
             normal.dot(&h) / normal.norm() / h_norm
         };
 
-        if approx_eq!(f64, self.specular_exponent, 1.0) {
-            self.specular_constant * n_dot_h
+        if approx_eq!(f64, self.params.specular_exponent, 1.0) {
+            self.params.specular_constant * n_dot_h
         } else {
-            self.specular_constant * n_dot_h.powf(self.specular_exponent)
+            self.params.specular_constant * n_dot_h.powf(self.params.specular_exponent)
         }
     }
 }
 
-// We cannot use a blanket impl<T: Lighting> Filter for T because we do
-// not want to make the Lighting trait public, so we use a macro
 macro_rules! impl_lighting_filter {
-    ($lighting_type:ty, $alpha_func:ident) => {
-        impl FilterEffect for $lighting_type {
-            fn render(
+    ($lighting_type:ty, $params_name:ident, $alpha_func:ident) => {
+        impl $params_name {
+            pub fn render(
                 &self,
-                node: &Node,
+                bounds_builder: BoundsBuilder,
                 ctx: &FilterContext,
                 acquired_nodes: &mut AcquiredNodes<'_>,
                 draw_ctx: &mut DrawingCtx,
-            ) -> Result<FilterResult, FilterError> {
-                let input = self.base.get_input(ctx, acquired_nodes, draw_ctx)?;
-                let mut bounds = self
-                    .base
-                    .get_bounds(ctx, node.parent().as_ref())?
-                    .add_input(&input)
-                    .into_irect(ctx, draw_ctx);
+            ) -> Result<FilterOutput, FilterError> {
+                let input_1 = ctx.get_input(
+                    acquired_nodes,
+                    draw_ctx,
+                    &self.params.in1,
+                    self.light.color_interpolation_filters,
+                )?;
+                let mut bounds: IRect = bounds_builder
+                    .add_input(&input_1)
+                    .compute(ctx)
+                    .clipped
+                    .into();
                 let original_bounds = bounds;
 
                 let scale = self
+                    .params
                     .kernel_unit_length
                     .map(|(dx, dy)| ctx.paffine().transform_distance(dx, dy));
 
-                let mut input_surface = input.surface().clone();
+                let mut input_surface = input_1.surface().clone();
 
                 if let Some((ox, oy)) = scale {
                     // Scale the input surface to match kernel_unit_length.
@@ -460,16 +482,12 @@ macro_rules! impl_lighting_filter {
 
                 let (ox, oy) = scale.unwrap_or((1.0, 1.0));
 
-                let light = Light::new(node, ctx.paffine())?;
-
-                // The generated color values are in the color space determined by
-                // color-interpolation-filters.
-                let surface_type = SurfaceType::from(light.color_interpolation_filters);
+                let source = self.light.source.transform(ctx.paffine());
 
                 let mut surface = ExclusiveImageSurface::new(
                     input_surface.width(),
                     input_surface.height(),
-                    surface_type,
+                    SurfaceType::from(self.light.color_interpolation_filters),
                 )?;
 
                 {
@@ -483,9 +501,10 @@ macro_rules! impl_lighting_filter {
 
                             let scaled_x = f64::from(x) * ox;
                             let scaled_y = f64::from(y) * oy;
-                            let z = f64::from(pixel.a) / 255.0 * self.surface_scale;
+                            let z = f64::from(pixel.a) / 255.0 * self.params.surface_scale;
 
-                            let (color, vector) = light.color_and_vector(scaled_x, scaled_y, z);
+                            let (color, vector) =
+                                self.light.color_and_vector(&source, scaled_x, scaled_y, z);
 
                             // compute the factor just once for the three colors
                             let factor = self.compute_factor(normal, vector);
@@ -625,15 +644,65 @@ macro_rules! impl_lighting_filter {
                     bounds = original_bounds;
                 }
 
-                Ok(FilterResult {
-                    name: self.base.result.clone(),
-                    output: FilterOutput { surface, bounds },
-                })
+                Ok(FilterOutput { surface, bounds })
             }
+        }
 
-            #[inline]
-            fn is_affected_by_color_interpolation_filters(&self) -> bool {
-                true
+        impl FilterEffect for $lighting_type {
+            fn resolve(&self, node: &Node) -> Result<(Primitive, PrimitiveParams), FilterError> {
+                let mut sources = node.children().rev().filter(|c| {
+                    c.is_element()
+                        && matches!(
+                            *c.borrow_element(),
+                            Element::FeDistantLight(_)
+                                | Element::FePointLight(_)
+                                | Element::FeSpotLight(_)
+                        )
+                });
+
+                let source_node = sources.next();
+                if source_node.is_none() || sources.next().is_some() {
+                    return Err(FilterError::InvalidLightSourceCount);
+                }
+
+                let source_node = source_node.unwrap();
+                let elt = source_node.borrow_element();
+
+                if elt.is_in_error() {
+                    return Err(FilterError::ChildNodeInError);
+                }
+
+                let source = match *elt {
+                    Element::FeDistantLight(ref l) => {
+                        UntransformedLightSource::Distant(l.element_impl.clone())
+                    }
+                    Element::FePointLight(ref l) => {
+                        UntransformedLightSource::Point(l.element_impl.clone())
+                    }
+                    Element::FeSpotLight(ref l) => {
+                        UntransformedLightSource::Spot(l.element_impl.clone())
+                    }
+                    _ => unreachable!(),
+                };
+
+                let cascaded = CascadedValues::new_from_node(node);
+                let values = cascaded.get();
+
+                Ok((
+                    self.base.clone(),
+                    PrimitiveParams::$params_name($params_name {
+                        params: self.params.clone(),
+                        light: Light {
+                            source,
+                            lighting_color: resolve_color(
+                                &values.lighting_color().0,
+                                UnitInterval::clamp(1.0),
+                                values.color().0,
+                            ),
+                            color_interpolation_filters: values.color_interpolation_filters(),
+                        },
+                    }),
+                ))
             }
         }
     };
@@ -647,8 +716,9 @@ fn specular_alpha(r: u8, g: u8, b: u8) -> u8 {
     max(max(r, g), b)
 }
 
-impl_lighting_filter!(FeDiffuseLighting, diffuse_alpha);
-impl_lighting_filter!(FeSpecularLighting, specular_alpha);
+impl_lighting_filter!(FeDiffuseLighting, DiffuseLighting, diffuse_alpha);
+
+impl_lighting_filter!(FeSpecularLighting, SpecularLighting, specular_alpha);
 
 /// 2D normal and factor stored separately.
 ///
@@ -890,3 +960,86 @@ impl Normal {
         )
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::document::Document;
+
+    #[test]
+    fn extracts_light_source() {
+        let document = Document::load_from_bytes(
+            br#"<?xml version="1.0" encoding="UTF-8"?>
+<svg xmlns="http://www.w3.org/2000/svg">
+  <filter id="filter">
+    <feDiffuseLighting id="diffuse_distant">
+      <feDistantLight azimuth="0.0" elevation="45.0"/>
+    </feDiffuseLighting>
+
+    <feSpecularLighting id="specular_point">
+      <fePointLight x="1.0" y="2.0" z="3.0"/>
+    </feSpecularLighting>
+
+    <feDiffuseLighting id="diffuse_spot">
+      <feSpotLight x="1.0" y="2.0" z="3.0"
+                   pointsAtX="4.0" pointsAtY="5.0" pointsAtZ="6.0"
+                   specularExponent="7.0" limitingConeAngle="8.0"/>
+    </feDiffuseLighting>
+  </filter>
+</svg>
+"#,
+        );
+
+        let node = document.lookup_internal_node("diffuse_distant").unwrap();
+        let lighting = borrow_element_as!(node, FeDiffuseLighting);
+        let (_, params) = lighting.resolve(&node).unwrap();
+        let diffuse_lighting = match params {
+            PrimitiveParams::DiffuseLighting(l) => l,
+            _ => unreachable!(),
+        };
+        assert_eq!(
+            diffuse_lighting.light.source,
+            UntransformedLightSource::Distant(FeDistantLight {
+                azimuth: 0.0,
+                elevation: 45.0,
+            })
+        );
+
+        let node = document.lookup_internal_node("specular_point").unwrap();
+        let lighting = borrow_element_as!(node, FeSpecularLighting);
+        let (_, params) = lighting.resolve(&node).unwrap();
+        let specular_lighting = match params {
+            PrimitiveParams::SpecularLighting(l) => l,
+            _ => unreachable!(),
+        };
+        assert_eq!(
+            specular_lighting.light.source,
+            UntransformedLightSource::Point(FePointLight {
+                x: 1.0,
+                y: 2.0,
+                z: 3.0,
+            })
+        );
+
+        let node = document.lookup_internal_node("diffuse_spot").unwrap();
+        let lighting = borrow_element_as!(node, FeDiffuseLighting);
+        let (_, params) = lighting.resolve(&node).unwrap();
+        let diffuse_lighting = match params {
+            PrimitiveParams::DiffuseLighting(l) => l,
+            _ => unreachable!(),
+        };
+        assert_eq!(
+            diffuse_lighting.light.source,
+            UntransformedLightSource::Spot(FeSpotLight {
+                x: 1.0,
+                y: 2.0,
+                z: 3.0,
+                points_at_x: 4.0,
+                points_at_y: 5.0,
+                points_at_z: 6.0,
+                specular_exponent: 7.0,
+                limiting_cone_angle: Some(8.0),
+            })
+        );
+    }
+}
