@@ -9,6 +9,7 @@ use crate::element::{ElementResult, SetAttributes};
 use crate::error::*;
 use crate::node::Node;
 use crate::parsers::{NonNegative, NumberOptionalNumber, Parse, ParseValue};
+use crate::property_defs::ColorInterpolationFilters;
 use crate::rect::IRect;
 use crate::surface_utils::{
     iterators::{PixelRectangle, Pixels},
@@ -17,44 +18,44 @@ use crate::surface_utils::{
 };
 use crate::xml::Attributes;
 
-use super::context::{FilterContext, FilterOutput, FilterResult};
-use super::{FilterEffect, FilterError, PrimitiveWithInput};
+use super::bounds::BoundsBuilder;
+use super::context::{FilterContext, FilterOutput};
+use super::{FilterEffect, FilterError, Input, Primitive, PrimitiveParams};
 
 /// Enumeration of the possible morphology operations.
+#[derive(Clone)]
 enum Operator {
     Erode,
     Dilate,
 }
 
+enum_default!(Operator, Operator::Erode);
+
 /// The `feMorphology` filter primitive.
+#[derive(Default)]
 pub struct FeMorphology {
-    base: PrimitiveWithInput,
+    base: Primitive,
+    params: Morphology,
+}
+
+/// Resolved `feMorphology` primitive for rendering.
+#[derive(Clone, Default)]
+pub struct Morphology {
+    in1: Input,
     operator: Operator,
     radius: (f64, f64),
 }
 
-impl Default for FeMorphology {
-    /// Constructs a new `Morphology` with empty properties.
-    #[inline]
-    fn default() -> FeMorphology {
-        FeMorphology {
-            base: PrimitiveWithInput::new::<Self>(),
-            operator: Operator::Erode,
-            radius: (0.0, 0.0),
-        }
-    }
-}
-
 impl SetAttributes for FeMorphology {
     fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
-        self.base.set_attributes(attrs)?;
+        self.params.in1 = self.base.parse_one_input(attrs)?;
 
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "operator") => self.operator = attr.parse(value)?,
+                expanded_name!("", "operator") => self.params.operator = attr.parse(value)?,
                 expanded_name!("", "radius") => {
                     let NumberOptionalNumber(NonNegative(x), NonNegative(y)) = attr.parse(value)?;
-                    self.radius = (x, y);
+                    self.params.radius = (x, y);
                 }
                 _ => (),
             }
@@ -64,35 +65,49 @@ impl SetAttributes for FeMorphology {
     }
 }
 
-impl FilterEffect for FeMorphology {
-    fn render(
+impl Morphology {
+    pub fn render(
         &self,
-        node: &Node,
+        bounds_builder: BoundsBuilder,
         ctx: &FilterContext,
         acquired_nodes: &mut AcquiredNodes<'_>,
         draw_ctx: &mut DrawingCtx,
-    ) -> Result<FilterResult, FilterError> {
-        let input = self.base.get_input(ctx, acquired_nodes, draw_ctx)?;
-        let bounds = self
-            .base
-            .get_bounds(ctx, node.parent().as_ref())?
-            .add_input(&input)
-            .into_irect(ctx, draw_ctx);
+    ) -> Result<FilterOutput, FilterError> {
+        // Although https://www.w3.org/TR/filter-effects/#propdef-color-interpolation-filters does not mention
+        // feMorphology as being one of the primitives that does *not* use that property,
+        // the SVG1.1 test for filters-morph-01-f.svg fails if we pass the value from the ComputedValues here (that
+        // document does not specify the color-interpolation-filters property, so it defaults to linearRGB).
+        // So, we pass Auto, which will get resolved to SRGB, and that makes that test pass.
+        //
+        // I suppose erosion/dilation doesn't care about the color space of the source image?
+
+        let input_1 = ctx.get_input(
+            acquired_nodes,
+            draw_ctx,
+            &self.in1,
+            ColorInterpolationFilters::Auto,
+        )?;
+        let bounds: IRect = bounds_builder
+            .add_input(&input_1)
+            .compute(ctx)
+            .clipped
+            .into();
 
         let (rx, ry) = self.radius;
         let (rx, ry) = ctx.paffine().transform_distance(rx, ry);
 
         // The radii can become negative here due to the transform.
-        let (rx, ry) = (rx.abs(), ry.abs());
+        // Additionally The radii being excessively large causes cpu hangups
+        let (rx, ry) = (rx.abs().min(10.0), ry.abs().min(10.0));
 
         let mut surface = ExclusiveImageSurface::new(
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-            input.surface().surface_type(),
+            input_1.surface().surface_type(),
         )?;
 
         surface.modify(&mut |data, stride| {
-            for (x, y, _pixel) in Pixels::within(input.surface(), bounds) {
+            for (x, y, _pixel) in Pixels::within(input_1.surface(), bounds) {
                 // Compute the kernel rectangle bounds.
                 let kernel_bounds = IRect::new(
                     (f64::from(x) - rx).floor() as i32,
@@ -114,9 +129,12 @@ impl FilterEffect for FeMorphology {
                     a: initial,
                 };
 
-                for (_x, _y, pixel) in
-                    PixelRectangle::within(&input.surface(), bounds, kernel_bounds, EdgeMode::None)
-                {
+                for (_x, _y, pixel) in PixelRectangle::within(
+                    &input_1.surface(),
+                    bounds,
+                    kernel_bounds,
+                    EdgeMode::None,
+                ) {
                     let op = match self.operator {
                         Operator::Erode => min,
                         Operator::Dilate => max,
@@ -132,18 +150,19 @@ impl FilterEffect for FeMorphology {
             }
         });
 
-        Ok(FilterResult {
-            name: self.base.result.clone(),
-            output: FilterOutput {
-                surface: surface.share()?,
-                bounds,
-            },
+        Ok(FilterOutput {
+            surface: surface.share()?,
+            bounds,
         })
     }
+}
 
-    #[inline]
-    fn is_affected_by_color_interpolation_filters(&self) -> bool {
-        false
+impl FilterEffect for FeMorphology {
+    fn resolve(&self, _node: &Node) -> Result<(Primitive, PrimitiveParams), FilterError> {
+        Ok((
+            self.base.clone(),
+            PrimitiveParams::Morphology(self.params.clone()),
+        ))
     }
 }
 
