@@ -7,6 +7,8 @@ use crate::element::{ElementResult, SetAttributes};
 use crate::error::*;
 use crate::node::{CascadedValues, Node};
 use crate::parsers::{NonNegative, NumberOptionalNumber, Parse, ParseValue};
+use crate::property_defs::ColorInterpolationFilters;
+use crate::rect::IRect;
 use crate::surface_utils::{
     shared_surface::{ExclusiveImageSurface, SurfaceType},
     ImageSurfaceDataExt, Pixel, PixelOps,
@@ -14,8 +16,9 @@ use crate::surface_utils::{
 use crate::util::clamp;
 use crate::xml::Attributes;
 
-use super::context::{FilterContext, FilterOutput, FilterResult};
-use super::{FilterEffect, FilterError, Primitive};
+use super::bounds::BoundsBuilder;
+use super::context::{FilterContext, FilterOutput};
+use super::{FilterEffect, FilterError, Primitive, PrimitiveParams};
 
 /// Enumeration of the tile stitching modes.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
@@ -24,6 +27,8 @@ enum StitchTiles {
     NoStitch,
 }
 
+enum_default!(StitchTiles, StitchTiles::NoStitch);
+
 /// Enumeration of the noise types.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
 enum NoiseType {
@@ -31,55 +36,67 @@ enum NoiseType {
     Turbulence,
 }
 
+enum_default!(NoiseType, NoiseType::Turbulence);
+
 /// The `feTurbulence` filter primitive.
+#[derive(Default)]
 pub struct FeTurbulence {
     base: Primitive,
+    params: Turbulence,
+}
+
+/// Resolved `feTurbulence` primitive for rendering.
+#[derive(Clone)]
+pub struct Turbulence {
     base_frequency: (f64, f64),
     num_octaves: i32,
     seed: i32,
     stitch_tiles: StitchTiles,
     type_: NoiseType,
+    color_interpolation_filters: ColorInterpolationFilters,
 }
 
-impl Default for FeTurbulence {
+impl Default for Turbulence {
     /// Constructs a new `Turbulence` with empty properties.
     #[inline]
-    fn default() -> FeTurbulence {
-        FeTurbulence {
-            base: Primitive::new::<Self>(),
+    fn default() -> Turbulence {
+        Turbulence {
             base_frequency: (0.0, 0.0),
             num_octaves: 1,
             seed: 0,
-            stitch_tiles: StitchTiles::NoStitch,
-            type_: NoiseType::Turbulence,
+            stitch_tiles: Default::default(),
+            type_: Default::default(),
+            color_interpolation_filters: Default::default(),
         }
     }
 }
 
 impl SetAttributes for FeTurbulence {
     fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
-        self.base.set_attributes(attrs)?;
+        self.base.parse_no_inputs(attrs)?;
 
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
                 expanded_name!("", "baseFrequency") => {
                     let NumberOptionalNumber(NonNegative(x), NonNegative(y)) = attr.parse(value)?;
-                    self.base_frequency = (x, y);
+                    self.params.base_frequency = (x, y);
                 }
                 expanded_name!("", "numOctaves") => {
-                    self.num_octaves = attr.parse(value)?;
+                    self.params.num_octaves = attr.parse(value)?;
                 }
                 // Yes, seed needs to be parsed as a number and then truncated.
                 expanded_name!("", "seed") => {
                     let v: f64 = attr.parse(value)?;
-                    self.seed = clamp(
+                    self.params.seed = clamp(
                         v.trunc(),
                         f64::from(i32::min_value()),
                         f64::from(i32::max_value()),
                     ) as i32;
                 }
-                expanded_name!("", "stitchTiles") => self.stitch_tiles = attr.parse(value)?,
-                expanded_name!("", "type") => self.type_ = attr.parse(value)?,
+                expanded_name!("", "stitchTiles") => {
+                    self.params.stitch_tiles = attr.parse(value)?
+                }
+                expanded_name!("", "type") => self.params.type_ = attr.parse(value)?,
                 _ => (),
             }
         }
@@ -323,18 +340,15 @@ impl NoiseGenerator {
     }
 }
 
-impl FilterEffect for FeTurbulence {
-    fn render(
+impl Turbulence {
+    pub fn render(
         &self,
-        node: &Node,
+        bounds_builder: BoundsBuilder,
         ctx: &FilterContext,
         _acquired_nodes: &mut AcquiredNodes<'_>,
-        draw_ctx: &mut DrawingCtx,
-    ) -> Result<FilterResult, FilterError> {
-        let bounds = self
-            .base
-            .get_bounds(ctx, node.parent().as_ref())?
-            .into_irect(ctx, draw_ctx);
+        _draw_ctx: &mut DrawingCtx,
+    ) -> Result<FilterOutput, FilterError> {
+        let bounds: IRect = bounds_builder.compute(ctx).clipped.into();
 
         let affine = ctx.paffine().invert().unwrap();
 
@@ -348,11 +362,9 @@ impl FilterEffect for FeTurbulence {
             f64::from(bounds.height()),
         );
 
-        let cascaded = CascadedValues::new_from_node(node);
-        let values = cascaded.get();
         // The generated color values are in the color space determined by
         // color-interpolation-filters.
-        let surface_type = SurfaceType::from(values.color_interpolation_filters());
+        let surface_type = SurfaceType::from(self.color_interpolation_filters);
 
         let mut surface = ExclusiveImageSurface::new(
             ctx.source_graphic().width(),
@@ -395,18 +407,22 @@ impl FilterEffect for FeTurbulence {
             }
         });
 
-        Ok(FilterResult {
-            name: self.base.result.clone(),
-            output: FilterOutput {
-                surface: surface.share()?,
-                bounds,
-            },
+        Ok(FilterOutput {
+            surface: surface.share()?,
+            bounds,
         })
     }
+}
 
-    #[inline]
-    fn is_affected_by_color_interpolation_filters(&self) -> bool {
-        true
+impl FilterEffect for FeTurbulence {
+    fn resolve(&self, node: &Node) -> Result<(Primitive, PrimitiveParams), FilterError> {
+        let cascaded = CascadedValues::new_from_node(node);
+        let values = cascaded.get();
+
+        let mut params = self.params.clone();
+        params.color_interpolation_filters = values.color_interpolation_filters();
+
+        Ok((self.base.clone(), PrimitiveParams::Turbulence(params)))
     }
 }
 
