@@ -6,9 +6,9 @@ use std::fmt;
 use std::io::BufReader;
 use std::path::{Path, PathBuf};
 
-use librsvg::surface_utils::compare_surfaces::BufferDiff;
 use librsvg::surface_utils::shared_surface::{SharedImageSurface, SurfaceType};
 
+use crate::compare_surfaces::BufferDiff;
 use crate::reference_utils::{surface_from_png, Compare, Deviation, Reference};
 
 /// Checks that the variable of type [u8] can be parsed as a PNG file.
